﻿using System;
using System.Collections.Generic;
using System.IO;
namespace BeeDevelopment.BbcBasicEditor {
	class Program {

		static string ReadAllText(string filename) {
			try {
				return BbcBasicFile.FromFile(filename).GetDataAsString();
			} catch {
				try {
					return Tokeniser.Detokenise(File.ReadAllBytes(filename));
				} catch {
					return File.ReadAllText(filename);
				}
			}
		}

		static void WriteAllText(string filename, string text) {
			var CurrentFile = new BbcBasicFile();
			CurrentFile.SetDataFromString(text ?? "");
			CurrentFile.Name = Path.GetFileNameWithoutExtension(filename).ToUpperInvariant();
			CurrentFile.Save(filename);
		}

		static void Main(string[] args) {
			if (args.Length == 0) {
				Console.Error.WriteLine("Nothing to do.");
				return;
			}
			var Commands = new Queue<string>(args);
			string WorkingFile = "";

			while (Commands.Count > 0) {
				var Command = Commands.Dequeue().ToLowerInvariant();
				if (!Command.StartsWith("-")) {
					Console.Error.WriteLine("Command name '{0}' does not start with a hyphen.", Command);
					return;
				}
				switch (Command) {
					case "-load":
						if (Commands.Count == 0) {
							Console.Error.WriteLine("Filename not specified for load command.");
							return;
						}
						try {
							WorkingFile = ReadAllText(Commands.Dequeue());
						} catch (Exception ex) {
							Console.Error.WriteLine("Error loading file: {0}", ex.Message);
							return;
						}
						break;
					case "-save":
						if (Commands.Count == 0) {
							Console.Error.WriteLine("Filename not specified for save command.");
							return;
						}
						try {
							WriteAllText(Commands.Dequeue(), WorkingFile);
						} catch (Exception ex) {
							Console.Error.WriteLine("Error saving file: {0}", ex.Message);
							return;
						}
						break;
					case "-renumber":
						try {
							var Data = Tokeniser.Tokenise(WorkingFile);
							int Start = 10, Step = 10;
							if (Commands.Count > 0 && !Commands.Peek().StartsWith("-")) {
								if (!int.TryParse(Commands.Dequeue(), out Start)) {
									Console.Error.WriteLine("Could not parse renumber start.");
									return;
								}
								if (Commands.Count > 0 && !Commands.Peek().StartsWith("-")) {
									if (!int.TryParse(Commands.Dequeue(), out Step)) {
										Console.Error.WriteLine("Could not parse renumber step.");
										return;
									}
								}
							}
							Tokeniser.Renumber(Data, Start, Step);
							WorkingFile = Tokeniser.Detokenise(Data);
						} catch (Exception ex) {
							Console.Error.WriteLine("Error renumbering file: {0}", ex.Message);
							return;
						}
						break;
					default:
						Console.Error.WriteLine("Unrecognised command '{0}'.", Command);
						return;
				}
			}		

		}
	}
}
